<?cs include:"doctype.cs" ?>
<?cs include:"macros.cs" ?>
<html>
<?cs include:"head_tag.cs" ?>
<body class="gc-documentation" itemscope itemtype="http://schema.org/Article">
<a name="top"></a>
<?cs call:custom_masthead() ?>

<div id="body-content">
<div id="doc-content" style="position:relative;">

<?cs call:tag_list(root.descr) ?>

<?cs include:"footer.cs" ?>
</div><!-- end doc-content -->

<?cs include:"trailer.cs" ?>

</body>
</html>



