.jd-toptitle {
    padding-left: 6px;
    margin-bottom: 30px;
    font-size: 160%;
    font-weight: bold;
}

div#jd-content table {
    border: none;
}

div#jd-content td, div#jd-content th {
    font-size: small;
}

div#jd-content table.jd-linktable {
    margin-top: 3px;
    border-spacing: 0;
}

div#jd-content p.jd-deprecated-warning {
    margin-top: 0;
    margin-bottom: 10px;
}

div#jd-content table.jd-linktable th {
    vertical-align: top;
    text-align: left;
    padding-top: 2px;
    padding-bottom: 2px;
    padding-left: 7px;
    padding-right: 7px;
    border: none;
    border-top: 1px solid #d2d7d0;
    background-color: #F7FCF4;
}

div#jd-content table.jd-linktable td {
    border: none;
}

div#jd-content table.jd-linktable td  p {
    padding: 0;
    margin: 0;
    line-height: 110%;
}

div#jd-content table.jd-linktable .jd-linkcol {
    vertical-align: top;
    padding-top: 3px;
    padding-bottom: 0;
    padding-left: 7px;
    padding-right: 7px;
    border-top: 1px solid #d2d7d0;
    background-color: #E5F1E0;
    line-height: 110%;
}

div#jd-content table.jd-linktable .jd-descrcol {
    vertical-align: top;
    padding-top: 3px;
    padding-bottom: 0;
    padding-left: 7px;
    padding-right: 7px;
    border-top: 1px solid #d2d7d0;
    background-color: #F7FCF4;
    line-height: 110%;
}

div#jd-content table.jd-linktable .jd-descrcol p {
    padding: 0;
    margin: 0;
    line-height: 110%;
}

div#jd-content table.jd-linktable .jd-valcol {
    vertical-align: top;
    padding-top: 3px;
    padding-bottom: 0;
    padding-left: 7px;
    padding-right: 7px;
    border-top: 1px solid #d2d7d0;
    background-color: #E5F1E0;
    line-height: 110%;
}

div#jd-content table.jd-linktable .jd-commentrow {
    vertical-align: top;
    padding-top: 3px;
    padding-bottom: 4px;
    padding-left: 7px;
    padding-right: 7px;
    background-color: #F7FCF4;
    line-height: 110%;
}

div#jd-content div.jd-inheritedlinks {
    vertical-align: top;
    margin-top: 9px;
    padding-left: 7px;
    padding-right: 7px;
    background-color: #F7FCF4;
    line-height: 110%;
}

div#jd-content .jd-page_title-prefix {
    padding-top: 2em;
    margin-bottom: -14pt;
}

div#jd-content {
    margin-left: 0;
    margin-right: 10px;
    margin-bottom: 0;
}

div#jd-content h1 {
    padding-left: 10px;
}

div#jd-content h2 {
    padding-left: 10px;
}

div#jd-content h4 {
    margin-top: 9px;
    margin-bottom: 1px;
}

div#jd-content .jd-descr h5 {
    margin-bottom: 8px;
}

div#jd-content .sidebox h3 {
    margin: 1em 0 0 0;
}

div#jd-content .jd-letterlist {
    margin-top: 20px;
    margin-bottom: 0;
}

div#jd-content .jd-lettertable {
    margin-top: 15px;
    margin-right: 10px;
}
div#jd-content .jd-letterentries {
	list-style: none;
	margin-left: 0;
}
div#jd-content .jd-letterentrycomments {
    color: gray;
}

div#jd-content table.jd-inheritance-table {
    margin-top: 0;
    margin-left: 10px;
    margin-right: 10px;
    border-spacing: 0;
}

div#jd-content table.jd-inheritance-table td {
    border: none;
    margin: 0;
    padding: 0;
    background-color: white;
}

div#jd-content table.jd-inheritance-table .jd-inheritance-space {
    width: 10px;
}

div#jd-content table.jd-inheritance-table .jd-inheritance-interface-cell {
    padding-left: 17px;
}

div#jd-content h4.jd-details-title {
    margin: 0;
    background-color: #E5F1E0;
    padding: 2px;
    padding-left: 10px;
    padding-right: 10px;
    margin-top: 15px;
}

div#jd-content .jd-details {
    margin-top: 0;
    margin-left: -10px;
}

div#jd-content .jd-details-descr {
    line-height: 120%;
    padding-left: 10px;
    padding-top: 10px;
    padding-right: 20px;
}

div#jd-content .jd-descr h5,
div#jd-content .jd-details h5 {
    font-style: normal;
    text-decoration: none;
    font-size: 120%;
}

div#jd-content .jd-descr {
    padding-top: 0;
}

div#jd-content .jd-tagdata {
    margin-top: 6px;
    margin-bottom: 6px;
}

div#jd-content .jd-tagtitle {
    margin-top: 0px;
}

div#jd-content .jd-tagtable {
    margin-top: 10px;
    border-spacing: 0;
}

div#jd-content .jd-tagtable th {
    background: white;
    padding-left: 10px;
    padding-right: 10px;
line-height: 120%;
}

div#jd-content .jd-tagtable th,
div#jd-content .jd-tagtable td {
line-height: 120%;
    border: none;
    margin: 0;
    text-align: left;
    padding-top: 0px;
    padding-bottom: 5px;
}

div#jd-content .Code,code,pre,samp,var {
    color: #004000;
}

div#jd-content pre.Code {
    padding-left: 20px;
}

/* XXX I would really like to apply font-size: 9pt only if var/samp
   is NOT inside of a .jd-descr div. */
div#jd-content .jd-descr code,var,samp {
    padding-left: 0px;
}

#search_autocomplete {
    font-size: 80%;
}

div#jd-searchbox table.jd-autocomplete-table-hidden {
    display: none;
}

div#jd-searchbox table.jd-autocomplete-table-showing {
    z-index: 10;
    border: 1px solid #3366cc;
    position: relative;
    top: -14px;
    left: 5px;
    background-color: white;
}

div#jd-searchbox td.jd-autocomplete {
    font-family: Arial, sans-serif;
    padding-left: 6px;
    padding-right: 6px;
    padding-top: 1px;
    padding-bottom: 1px;
    font-size: 80%;
    border: none;
    margin: 0;
    line-height: 105%;
}

div#jd-searchbox td.jd-selected {
    background-color: #E5F1E0;
}

div#jd-searchbox td.jd-linktype {
    color: #999999;
}

div#jd-content .jd-expando-trigger {
    margin-left: -8px;
    margin-right: 0px;
    border: none;
}

div#jd-build-id {
    color: #666;
    width: 100%;
    text-align: right;
    padding-right: 5px;
    padding-bottom: 3px;
}

@media print {
    #jd-searchbox, .jd-nav {
        display: none;
    }
    div#jd-content {
        margin-top: 0px;
    }
}

